
uniform sampler3D u_img3_vol ;
uniform sampler3D u_img3_flg ;
uniform sampler3D u_img3_mask;
uniform sampler1D u_img1_tf  ;
uniform sampler1D u_img1_psu ;

uniform float     u_alpha      ;
uniform int       u_doPsuedo   ;
uniform vec4      u_texCdOfst  ; //(x,y,z,0)
uniform vec4      u_eyePos     ;

uniform vec3      u_center     ;
uniform float     u_radi       ;
uniform float     u_minPhi     ;
uniform float     u_maxPhi     ;

varying vec3 globalPos;

#define M_PI 3.1415926535897932384626433832795



vec3 calcGradDir( const float pivC, vec3  texCd     )
{
    float px = pivC - texture3D( u_img3_vol, texCd + u_texCdOfst.xww);
    float py = pivC - texture3D( u_img3_vol, texCd + u_texCdOfst.wxw);
    float pz = pivC - texture3D( u_img3_vol, texCd + u_texCdOfst.wwx);
    return normalize( vec3( px, py,pz ) );
}




const vec3  lightPos = vec3( 5000, 5000, 1000);
const vec3  Kd       = vec3( 1   , 1   , 1   );
const vec3  Ks       = vec3( 0.6 , 0.6 , 0.6 );
const float Shin     = 128;

vec3  phogShading( const vec3 objPos, const vec3 baseColor, const vec3 N)
{
    vec3 L = normalize( lightPos - objPos );
    vec3 V = normalize( u_eyePos - objPos );
    vec3 R = normalize( 2 * dot( L, N ) * N - L );

    float diffK =      abs( dot(N,L) )        ;
    float specK = pow( abs( dot(R,V) ), Shin );

    return (diffK + 0.2) * baseColor + specK * Ks ; 
}




const float COEF_1_255     = 0.00392;
const float COEF_1_255_2   = 0.00196;
const float THRESH_NO_MASK = COEF_1_255  ;
const float THRESH_FORE    = COEF_1_255 * 254.0 + COEF_1_255_2;


void main(void)
{
	float3 v = globalPos - u_center;
	float  L = length(v) ;
	float  r = L / u_radi;

	v = v / L;
	float phi = (asin(v.z) - u_minPhi) / (u_maxPhi - u_minPhi);
    float the = (atan(v.y, v.x) + M_PI) / (2*M_PI);

	if( r < 0 || r > 1 || phi < 0 || phi > 1 || the < 0 || the > 1 ) return;


	float3 tex = float3(r,the,phi);
    float imgI = texture3D(u_img3_vol, tex).x;
	
	float tf1   = texture1D( u_img1_tf, imgI).x;

    if( u_doPsuedo ) gl_FragColor.xyz = texture1D( u_img1_psu, imgI ).xyz;
    else           	 gl_FragColor.xyz = vec3( imgI, imgI, imgI );
	
	mat3 M = mat3(cos(the), sin(the), 0, 
	             -sin(the), cos(the), 0,
		          0,0,1);

    vec3 N = M * calcGradDir( imgI, tex ) ;

    gl_FragColor.xyz = phogShading( globalPos, gl_FragColor.xyz, N );
    gl_FragColor.w   = u_alpha * tf1 ;
}





